/*
   Random Events
   Created by Randall Maas, 2009
   Copyright (c) 2008-2009, Randall Maas
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   Neither the name of Blackwood Designs, LLC nor the names of its contributors
   may be used to endorse or promote products derived from this software without
   specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
   THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System . Threading;
using System . Windows . Forms;
using System . Diagnostics;

static partial class Program
{
   /// <summary>
   /// Random number generator
   /// </summary>
   static Random Rnd =  new Random();

   static int Nesting=0;
   static int MaxKey = 0x5a;

   // Probability figures
   const double R1 = 0.1;
   const double R2 = 0.5;
   const double R3 = 0.7;
   const double R4 = 0.9;

   /// <summary>
   /// Sends random events to passed application
   /// </summary>
   /// <param name="Port">The window / etc to send events to</param>
   static void SendRandomEvents(ISendPort Port)
   {
      Nesting++;
      if (Nesting > 10 || 0 != Shutdown) {Nesting--; return;}

      Application . DoEvents();
      KEYBDINPUT KEvent = new KEYBDINPUT();
      MOUSEINPUT MEvent = new MOUSEINPUT();

      // Todo: Record these events
      double R = Rnd . NextDouble();
      if (R1 > R)
        ; // exit
      else if (R2 > R)
        {
           // Send a key event
           // Fill out the event structure
           KEvent . dwFlags = 0;
           KEvent . KeyDown((short) Rnd . Next(MaxKey)); // Find a key to press
           KEvent . dwExtraInfo = Port . DefaultExtraInfo();

           Log . WriteLine ("{0},{1},{2},{3} \n", BaseTime-Stopwatch.GetTimestamp(), KbdPress, KEvent . wVk, KEvent . dwExtraInfo);
           Port . Send(KEvent);

           // Recurse to send other events while this is down
           SendRandomEvents(Port);

           KEvent . dwFlags = KEYEVENTF . KEYUP;
           Log . WriteLine ("{0},{1},{2},{3}\n", BaseTime-Stopwatch.GetTimestamp(), KbdRelease, KEvent . wVk, KEvent . dwExtraInfo);
           Port . Send(KEvent);
        }
      else if (R3 > R)
        {
           // Send mouse click
           MEvent . dwFlags = (MOUSEEVENTF) (1 << (2*Rnd . Next(0, 3)+1)) ;
           MEvent . dwExtraInfo = Port . DefaultExtraInfo();
           Log . WriteLine("{0},{1},{2}\n", BaseTime-Stopwatch.GetTimestamp(), MouseClick, MEvent . dwFlags);
           Port . Send(MEvent);

           // Recurse to send other events while this is down
           SendRandomEvents(Port);

           MEvent .  dwFlags = (MOUSEEVENTF)((int) MEvent .  dwFlags << 1);
           MEvent . dwExtraInfo = Port . DefaultExtraInfo();
           Log . WriteLine("{0},{1},{2}\n", BaseTime-Stopwatch.GetTimestamp(), MouseClick, MEvent . dwFlags);
           Port . Send(MEvent);
        }
       else
        {
           // Send mouse move
           MEvent .  dwFlags = MOUSEEVENTF . ABSOLUTE | MOUSEEVENTF . MOVE;
           MEvent .  dx = Rnd . Next(WinRect . Left, WinRect . Right) ;
           MEvent .  dy = Rnd . Next(WinRect . Top,  WinRect . Bottom);
           MEvent . dwExtraInfo = Port . DefaultExtraInfo();
           Log . WriteLine("{0},{1},{2},{3}\n", BaseTime-Stopwatch.GetTimestamp(), MouseMove, WinRect . Left - MEvent . dx, WinRect . Top - MEvent . dy);
           Port . Send(MEvent);
        }

      Thread . Sleep (20);
      Nesting--;
   }
}

