﻿/*
   Generic message sending
   Created by Randall Maas, Wednesday October 13 2009
   Copyright (c) 2009, Randall Maas
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   Neither the name of Blackwood Designs, LLC nor the names of its contributors
   may be used to endorse or promote products derived from this software without
   specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
   THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;

delegate void SendResponse(IntPtr Ret);

/// <summary>
/// An interface used to represent different things that may acts as ports
/// for messages (events)
/// </summary>
interface ISendPort
{
   /// <summary>
   /// Default extra information the caller can place into an event message
   /// </summary>
   /// <returns></returns>
   IntPtr DefaultExtraInfo();

   /// <summary>
   /// Sends the specified keyboard event to the current focus window
   /// </summary>
   /// <param name="Event">Keyboard Event to Send</param>
   /// <returns>True if the message was sent, false if an error sending</returns>
   bool Send(KEYBDINPUT Event);

   /// <summary>
   /// Sends the specified Mouse event to the current focus window
   /// </summary>
   /// <param name="Event">Mouse Event to Send</param>
   /// <returns>True if the message was sent, false if an error sending</returns>
   bool Send(MOUSEINPUT Event);

   /// <summary>
   /// Sends the specified Hardare event to the current focus window
   /// </summary>
   /// <param name="Event">Hardware Event to Send</param>
   /// <returns>True if the message was sent, false if an error sending</returns>
   bool Send(HARDWAREINPUT Event);

   /// <summary>
   /// Sends the specified message
   /// </summary>
   /// <param name="Msg"></param>
   /// <param name="Param1"></param>
   /// <param name="Param2"></param>
   /// <returns>True if the message was sent, false if an error sending</returns>
   bool Send(WM Msg, IntPtr Param1, IntPtr Param2);

   /// <summary>
   /// Sends the specified message
   /// </summary>
   /// <param name="Msg"></param>
   /// <param name="Param1"></param>
   /// <param name="Param2"></param>
   /// <param name="Result">The callback to receive the response</param>
   /// <returns>True if the message was sent, false if an error sending</returns>
   bool Send(WM Msg, IntPtr Param1, IntPtr Param2, SendResponse Result);
}
