/*
   Generic message sending
   Created by Randall Maas, 2009
   Portions from pinvoke.net
   Copyright (c) 2009, Randall Maas
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   Neither the name of Blackwood Designs, LLC nor the names of its contributors
   may be used to endorse or promote products derived from this software without
   specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
   THE POSSIBILITY OF SUCH DAMAGE.
*/

using System ;
using System . Runtime . InteropServices;


/// <summary>
/// A helper structure used with 
/// </summary>
[Flags]
public enum KEYEVENTF : short
{
   EXTENDEDKEY = 0x01,
   KEYUP       = 0x02
}


/// <summary>
/// A structure that represent a keyboard button press / release event
/// </summary>
/// <seealso cref="http://pinvoke.net/default.aspx/Structures.INPUT" />
[StructLayout(LayoutKind.Sequential)]
public struct KEYBDINPUT
{
   /// <summary>
   /// The virutal key code
   /// </summary>
   public short  wVk;

   /// <summary>
   /// The keyboard scan code
   /// </summary>
   public short  wScan;

   /// <summary>
   /// Flags indicate whether it is a button release, and whether it is an AT extended key
   /// </summary>
   public KEYEVENTF dwFlags;

   /// <summary>
   /// Time of the keypress/release
   /// </summary>
   public int    time;

   /// <summary>
   /// Other information
   /// </summary>
   public IntPtr dwExtraInfo;

   /// <summary>
   /// Prepare a keyboard button press/release event
   /// </summary>
   /// <param name="VK">Key pressed</param>
   internal void KeyDown(short VK)
   {
      wVk         = VK;
      // Handle Extended key events, properly marking the event
      if ((VK >= 33 && VK <= 46) || (VK >= 91 && VK <= 93))
        dwFlags |= KEYEVENTF . EXTENDEDKEY;
//      dwExtraInfo = Win32 . GetMessageExtraInfo();
//      time        = 0; //GetTickCount() may be the correct one
   }
   // Send a second one with dwFlags |= KEYEVENTF . KEYUP
}

