/*
   Mouse movement
   created by Randall Maas  2008
   Portions from pinvoke.net
   Copyright (c) 2008-2009, Randall Maas
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.

   Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   Neither the name of Blackwood Designs, LLC nor the names of its contributors
   may be used to endorse or promote products derived from this software without
   specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
   THE POSSIBILITY OF SUCH DAMAGE.
*/

using System ;
using System . Runtime . InteropServices;

/// <summary>
/// A structure for mouse eventsevents
/// </summary>
/// <seealso cref="http://pinvoke.net/default.aspx/Structures.INPUT" />
[StructLayout(LayoutKind.Sequential)]
struct MOUSEINPUT
{
   /// <summary>
   /// The X coordinate for the new position or change in position
   /// </summary>
   public int    dx;

   /// <summary>
   /// The Y coordinate for the new position or change in position
   /// </summary>
   public int    dy;

   /// <summary>
   /// 
   /// </summary>
   public int    mouseData;

   /// <summary>
   /// Flags indicate whether it is a button press, button release, a move,etc
   /// </summary>
   public MOUSEEVENTF dwFlags;

   /// <summary>
   /// Time of the keypress/release
   /// </summary>
   public int    time;

   /// <summary>
   /// Other information
   /// </summary>
   public IntPtr dwExtraInfo;
}

/// <summary>
/// Flags for the different kinds of mouse events -- movement, button press, etc.
/// </summary>
/// <seealso cref="http://pinvoke.net/default.aspx/Structures.INPUT" />
[Flags]
enum MOUSEEVENTF : int
{
   MOVE       = 0x01,
   LEFTDOWN   = 0x02,
   LEFTUP     = 0x04,
   RIGHTDOWN  = 0x08,
   RIGHTUP    = 0x10,
   MIDDLEDOWN = 0x20,
   MIDDLEUP   = 0x40,
   ABSOLUTE   = 0x8000
}
